--------------------------------------------------------------------------------
-- Company: 
-- Engineer:
--
-- Create Date:    03:19:50 10/28/05
-- Design Name:    
-- Module Name:    setup - Behavioral
-- Project Name:   
-- Target Device:  
-- Tool versions:  
-- Description:
--
-- Dependencies:
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
library UNISIM;
use UNISIM.VComponents.all;

entity setup is
  port (
    CLK : in std_logic;
    LED : out std_logic_vector(7 downto 0);
    AA : out std_logic_vector(19 downto 0);
    ADVLDA : out std_logic;
    XRWA : out std_logic;
    CEA : out std_logic;
    BWA : out std_logic_vector(3 downto 0);
    CELA1X : out std_logic;
    CEHA1X : out std_logic;
    CEA2X : out std_logic;
    CEA2 : out std_logic;
    CLKMA0 : out std_logic;
    CLKMA1 : out std_logic;
    IOA : inout std_logic_vector(31 downto 0);
    IOPA : inout std_logic_vector(3 downto 0);
    LBOA : out std_logic;
    OEAX : out std_logic;
    ZZA : out std_logic;
    USBD : inout std_logic_vector(7 downto 0);
    USBRDX : out std_logic;
    USBWR : out std_logic;
    USBTXEX : in std_logic;
    USBRXFX : in std_logic;
    USBRSTX : out std_logic);
end setup;

architecture Behavioral of setup is
  signal RST : std_logic;
  signal BCLK : std_logic;
  signal CNT : std_logic_vector(4 downto 0);
  signal A : std_logic_vector(19 downto 0);
  signal D : std_logic_vector(7 downto 0);
  signal RDYX : std_logic;
  signal B : std_logic_vector(3 downto 0);

begin

  LED <= not A(19 downto 12);
  AA <= A;
  ADVLDA <= '0';
  XRWA <= '0';
  CEA <= '0';
  BWA <= B;
  CELA1X <= '0';
  CEHA1X <= '0';
  CEA2X <= '0';
  CEA2 <= '1';
  CLKMA0 <= BCLK;
  CLKMA1 <= BCLK;
  IOA <= D & D & D & D;
  LBOA <= 'X';
  OEAX <= '0';
  ZZA <= '0';
  USBRSTX <= not RST;
  USBRDX <= RDYX;
  USBWR <= '0';
  
  IBG : IBUFG port map (
    I => CLK,
    O => BCLK);

  RC : ROC port map (
    O => RST);

  C: process (BCLK, RST)
  begin  -- process C
    if RST = '1' then               -- asynchronous reset (active high)
      CNT <= "00000";
      
    elsif BCLK'event and BCLK = '1' then  -- rising clock edge
      if CNT(2 downto 0) /= "000" or USBRXFX = '0' then
        CNT <= CNT + 1;
        
      end if;
      
    end if;
  end process C;

  AC: process (BCLK, RST)
  begin  -- process AC
    if RST = '1' then               -- asynchronous reset (active high)
      A <= "00000000000000000000";
      
    elsif BCLK'event and BCLK = '1' then  -- rising clock edge
      if CNT = "11111" then
        A <= A + 1;
        
      end if;
      
    end if;
  end process AC;

  I: process (BCLK, RST)
  begin  -- process I
    if RST = '1' then                   -- asynchronous reset (active high)
      RDYX <= '1';
      
    elsif BCLK'event and BCLK = '1' then  -- rising clock edge
      case CNT(2 downto 0) is
        when "001" => RDYX <= '0';
        when "100" => D <= USBD; RDYX <= '1';
        when others => null;        
      end case;
    end if;
  end process I;
      
  O: process (BCLK, RST)
  begin  -- process O
    if RST = '1' then                   -- asynchronous reset (active high)
      
    elsif BCLK'event and BCLK = '1' then  -- rising clock edge
      case CNT(4 downto 3) is
        when "00" => B <= "0111";
        when "01" => B <= "1011";
        when "10" => B <= "1101";
        when "11" => B <= "1110";
        when others => null;
      end case;
      
    end if;
  end process O;

end Behavioral;
